@@ -61,6 +61,42 @@ class AttributeType(Enum):
     CUSTOM_ATTRIBUTE                 = 'Custom Attribute'
 
 
+class ConformanceClause(Enum):
+    """
+    The specification of KMIP features supported by KMIP clients and servers.
+
+    The conformance clause defines the KMIP objects and operations that need
+    to be supported for a specific feature set. The clause is one of two core
+    components that make up a KMIP client/server profile. For more information,
+    see Section 5 of the KMIP 1.1 profiles document.
+    """
+    DISCOVER_VERSIONS                      = 1
+    BASELINE                               = 2
+    SECRET_DATA                            = 3
+    SYMMETRIC_KEY_STORE                    = 4
+    SYMMETRIC_KEY_FOUNDRY                  = 5
+    ASYMMETRIC_KEY_STORE                   = 6
+    ASYMMETRIC_KEY_AND_CERTIFICATE_STORE   = 7
+    ASYMMETRIC_KEY_FOUNDRY                 = 8
+    CERTIFICATE                            = 9
+    ASYMMETRIC_KEY_FOUNDRY_AND_CERTIFICATE = 10
+    STORAGE                                = 11
+
+
+class AuthenticationSuite(Enum):
+    """
+    The type of authentication suite used by KMIP clients and servers.
+
+    The authentication suite defines the protocol versions and cipher suites
+    that should be used to secure KMIP client/server communications. An
+    authentication suite is one of two core components that make up a KMIP
+    client/server profile. For more information, see Section 3 of the KMIP
+    1.1 profiles document.
+    """
+    BASIC = 1
+    TLS12 = 2
+
+
 # 9.1.1.2
 class Types(Enum):
     DEFAULT      = 0x00
@@ -313,7 +349,12 @@ class WrappingMethod(Enum):
 
 
 # 9.1.3.2.6
-class CertificateType(Enum):
+class CertificateTypeEnum(Enum):
+    """
+    The type of a Certificate Managed Object.
+
+    For more information, see Section 2.2.1 of the KMIP 1.1 specification.
+    """
     X_509 = 0x00000001
     PGP   = 0x00000002
 
@@ -330,10 +371,11 @@ class SecretDataType(Enum):
     PASSWORD = 0x00000001
     SEED     = 0x00000002
 
-
 # 9.1.3.2.10
 class OpaqueDataType(Enum):
-    pass
+    NONE = 0x80000000 # Not defined by the standard, but we need something.
+                      # The standard does say that values starting 0x8xxxxxx
+                      # are considered extensions
 
 
 # 9.1.3.2.11
@@ -458,6 +500,15 @@ class KeyRoleType(Enum):
     PVKOTH    = 0x00000015
 
 
+# 9.1.3.2.24
+class QueryFunction(Enum):
+    QUERY_OPERATIONS             = 0x00000001
+    QUERY_OBJECTS                = 0x00000002
+    QUERY_SERVER_INFORMATION     = 0x00000003
+    QUERY_APPLICATION_NAMESPACES = 0x00000004
+    QUERY_EXTENSION_LIST         = 0x00000005
+    QUERY_EXTENSION_MAP          = 0x00000006
+
 # 9.1.3.2.27
 class Operation(Enum):
     CREATE               = 0x00000001
@@ -569,3 +620,12 @@ class ObjectGroupMember(Enum):
 class StorageStatusMask(Enum):
     ONLINE_STORAGE      = 0x00000001
     ARCHIVAL_STORAGE    = 0x00000002
+
+class RevocationReasonCode(Enum):
+    UNSPECIFIED            = 0x00000001
+    KEY_COMPROMISE         = 0x00000002
+    CA_COMPROMISE          = 0x00000003
+    AFFILIATION_CHANGED    = 0x00000004
+    SUPERSEDED             = 0x00000005
+    CESSATION_OF_OPERATION = 0x00000006
+    PRIVILEGE_WITHDRAWN    = 0x00000007
