@@ -13,19 +13,22 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from six.moves import xrange
+
 from kmip.core import attributes
 from kmip.core.attributes import CryptographicParameters
 
 from kmip.core.factories.attribute_values import AttributeValueFactory
-from kmip.core.factories.keys import KeyFactory
 
 from kmip.core import enums
 from kmip.core.enums import AttributeType
 from kmip.core.enums import Tags
 from kmip.core.enums import Types
 from kmip.core.enums import CredentialType
+from kmip.core.enums import RevocationReasonCode as RevocationReasonCodeEnum
 
 from kmip.core.errors import ErrorStrings
+from kmip.core.misc import KeyFormatType
 
 from kmip.core.primitives import Struct
 from kmip.core.primitives import TextString
@@ -43,18 +46,20 @@ class Attribute(Struct):
     class AttributeName(TextString):
 
         def __init__(self, value=None):
-            super(self.__class__, self).__init__(value, Tags.ATTRIBUTE_NAME)
+            super(Attribute.AttributeName, self).__init__(
+                value, Tags.ATTRIBUTE_NAME)
 
     class AttributeIndex(Integer):
 
         def __init__(self, value=None):
-            super(self.__class__, self).__init__(value, Tags.ATTRIBUTE_INDEX)
+            super(Attribute.AttributeIndex, self).__init__(
+                value, Tags.ATTRIBUTE_INDEX)
 
     def __init__(self,
                  attribute_name=None,
                  attribute_index=None,
                  attribute_value=None):
-        super(self.__class__, self).__init__(tag=Tags.ATTRIBUTE)
+        super(Attribute, self).__init__(tag=Tags.ATTRIBUTE)
 
         self.value_factory = AttributeValueFactory()
 
@@ -66,7 +71,7 @@ class Attribute(Struct):
             attribute_value.tag = Tags.ATTRIBUTE_VALUE
 
     def read(self, istream):
-        super(self.__class__, self).read(istream)
+        super(Attribute, self).read(istream)
         tstream = BytearrayStream(istream.read(self.length))
 
         # Read the name of the attribute
@@ -106,9 +111,25 @@ class Attribute(Struct):
 
         # Write the length and value of the attribute
         self.length = tstream.length()
-        super(self.__class__, self).write(ostream)
+        super(Attribute, self).write(ostream)
         ostream.write(tstream.buffer)
 
+    def __eq__(self, other):
+        if isinstance(other, Attribute):
+            if self.attribute_name != other.attribute_name:
+                return False
+            elif self.attribute_index != other.attribute_index:
+                return False
+            elif self.attribute_value != other.attribute_value:
+                return False
+            else:
+                return True
+        else:
+            return NotImplemented
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
 
 # 2.1.2
 class Credential(Struct):
@@ -118,26 +139,32 @@ class Credential(Struct):
         ENUM_TYPE = CredentialType
 
         def __init__(self, value=None):
-            super(self.__class__, self).__init__(value, Tags.CREDENTIAL_TYPE)
+            super(Credential.CredentialType, self).__init__(
+                value, Tags.CREDENTIAL_TYPE)
 
     class UsernamePasswordCredential(Struct):
 
         class Username(TextString):
             def __init__(self, value=None):
-                super(self.__class__, self).__init__(value, Tags.USERNAME)
+                super(Credential.UsernamePasswordCredential.Username,
+                      self).__init__(
+                    value, Tags.USERNAME)
 
         class Password(TextString):
             def __init__(self, value=None):
-                super(self.__class__, self).__init__(value, Tags.PASSWORD)
+                super(Credential.UsernamePasswordCredential.Password,
+                      self).__init__(
+                    value, Tags.PASSWORD)
 
         def __init__(self, username=None, password=None):
-            super(self.__class__, self).__init__(tag=Tags.CREDENTIAL_VALUE)
+            super(Credential.UsernamePasswordCredential, self).__init__(
+                tag=Tags.CREDENTIAL_VALUE)
             self.username = username
             self.password = password
             self.validate()
 
         def read(self, istream):
-            super(self.__class__, self).read(istream)
+            super(Credential.UsernamePasswordCredential, self).read(istream)
             tstream = BytearrayStream(istream.read(self.length))
 
             # Read the username of the credential
@@ -161,7 +188,7 @@ class Credential(Struct):
 
             # Write the length and value of the credential
             self.length = tstream.length()
-            super(self.__class__, self).write(ostream)
+            super(Credential.UsernamePasswordCredential, self).write(ostream)
             ostream.write(tstream.buffer)
 
         def validate(self):
@@ -190,7 +217,7 @@ class Credential(Struct):
         class NetworkIdentifier(TextString):
 
             def __init__(self, value=None):
-                super(Credential.DeviceCredential.NetworkIdetifier, self).\
+                super(Credential.DeviceCredential.NetworkIdentifier, self).\
                     __init__(value, Tags.NETWORK_IDENTIFIER)
 
         class MachineIdentifier(TextString):
@@ -212,16 +239,17 @@ class Credential(Struct):
                      network_identifier=None,
                      machine_identifier=None,
                      media_identifier=None):
-            super(self.__class__, self).__init__(tag=Tags.CREDENTIAL_VALUE)
-            super.device_serial_number = device_serial_number
-            super.password = password
-            super.device_identifier = device_identifier
-            super.network_identifier = network_identifier
-            super.machine_identifier = machine_identifier
-            super.media_identifier = media_identifier
+            super(Credential.DeviceCredential, self).__init__(
+                tag=Tags.CREDENTIAL_VALUE)
+            self.device_serial_number = device_serial_number
+            self.password = password
+            self.device_identifier = device_identifier
+            self.network_identifier = network_identifier
+            self.machine_identifier = machine_identifier
+            self.media_identifier = media_identifier
 
         def read(self, istream):
-            super(self.__class__, self).read(istream)
+            super(Credential.DeviceCredential, self).read(istream)
             tstream = BytearrayStream(istream.read(self.length))
 
             # Read the password if it is next
@@ -275,19 +303,19 @@ class Credential(Struct):
 
             # Write the length and value of the credential
             self.length = tstream.length()
-            super(self.__class__, self).write(ostream)
+            super(Credential.DeviceCredential, self).write(ostream)
             ostream.write(tstream.buffer)
 
         def validate(self):
             pass
 
     def __init__(self, credential_type=None, credential_value=None):
-        super(self.__class__, self).__init__(tag=Tags.CREDENTIAL)
+        super(Credential, self).__init__(tag=Tags.CREDENTIAL)
         self.credential_type = credential_type
         self.credential_value = credential_value
 
     def read(self, istream):
-        super(self.__class__, self).read(istream)
+        super(Credential, self).read(istream)
         tstream = BytearrayStream(istream.read(self.length))
 
         # Read the type of the credential
@@ -315,7 +343,7 @@ class Credential(Struct):
 
         # Write the length and value of the credential
         self.length = tstream.length()
-        super(self.__class__, self).write(ostream)
+        super(Credential, self).write(ostream)
         ostream.write(tstream.buffer)
 
     def validate(self):
@@ -325,18 +353,12 @@ class Credential(Struct):
 # 2.1.3
 class KeyBlock(Struct):
 
-    class KeyFormatType(Enumeration):
-        ENUM_TYPE = enums.KeyFormatType
-
-        def __init__(self, value=None):
-            super(self.__class__, self).__init__(value, Tags.KEY_FORMAT_TYPE)
-
     class KeyCompressionType(Enumeration):
         ENUM_TYPE = enums.KeyCompressionType
 
         def __init__(self, value=None):
-            super(self.__class__, self).__init__(value,
-                                                 Tags.KEY_COMPRESSION_TYPE)
+            super(KeyBlock.KeyCompressionType, self).__init__(
+                value, Tags.KEY_COMPRESSION_TYPE)
 
     def __init__(self,
                  key_format_type=None,
@@ -345,7 +367,7 @@ class KeyBlock(Struct):
                  cryptographic_algorithm=None,
                  cryptographic_length=None,
                  key_wrapping_data=None):
-        super(self.__class__, self).__init__(Tags.KEY_BLOCK)
+        super(KeyBlock, self).__init__(Tags.KEY_BLOCK)
         self.key_format_type = key_format_type
         self.key_compression_type = key_compression_type
         self.key_value = key_value
@@ -355,18 +377,17 @@ class KeyBlock(Struct):
         self.validate()
 
     def read(self, istream):
-        super(self.__class__, self).read(istream)
+        super(KeyBlock, self).read(istream)
         tstream = BytearrayStream(istream.read(self.length))
 
-        self.key_format_type = KeyBlock.KeyFormatType()
+        self.key_format_type = KeyFormatType()
         self.key_format_type.read(tstream)
-        key_format_type = self.key_format_type.enum
 
         if self.is_tag_next(Tags.KEY_COMPRESSION_TYPE, tstream):
             self.key_compression_type = KeyBlock.KeyCompressionType()
             self.key_compression_type.read(tstream)
 
-        self.key_value = KeyValue(key_format_type=key_format_type)
+        self.key_value = KeyValue()
         self.key_value.read(tstream)
 
         if self.is_tag_next(Tags.CRYPTOGRAPHIC_ALGORITHM, tstream):
@@ -403,7 +424,7 @@ class KeyBlock(Struct):
 
         # Write the length and value of the credential
         self.length = tstream.length()
-        super(self.__class__, self).write(ostream)
+        super(KeyBlock, self).write(ostream)
         ostream.write(tstream.buffer)
 
     def validate(self):
@@ -411,9 +432,9 @@ class KeyBlock(Struct):
 
     def __validate(self):
         if self.key_format_type is not None:
-            if type(self.key_format_type) is not KeyBlock.KeyFormatType:
+            if type(self.key_format_type) is not KeyFormatType:
                 member = 'KeyBlock.key_format_type'
-                exp_type = KeyBlock.KeyFormatType
+                exp_type = KeyFormatType
                 rcv_type = type(self.key_format_type)
                 msg = ErrorStrings.BAD_EXP_RECV.format(member, 'type',
                                                        exp_type, rcv_type)
@@ -421,35 +442,78 @@ class KeyBlock(Struct):
 
 
 # 2.1.4
-class KeyValueString(ByteString):
+class KeyMaterial(ByteString):
 
     def __init__(self, value=None):
-        super(self.__class__, self).__init__(value, Tags.KEY_VALUE)
+        super(KeyMaterial, self).__init__(value, Tags.KEY_MATERIAL)
 
 
-class KeyValueStruct(Struct):
+# TODO (peter-hamilton) Get rid of this and replace with a KeyMaterial factory.
+class KeyMaterialStruct(Struct):
+
+    def __init__(self):
+        super(KeyMaterialStruct, self).__init__(Tags.KEY_MATERIAL)
+
+        self.data = BytearrayStream()
 
-    def __init__(self,
-                 key_format_type=None,
-                 key_material=None,
-                 attributes=None):
-        super(self.__class__, self).__init__(Tags.KEY_VALUE)
-        self.key_format_type = key_format_type
-        self.key_material = key_material
-        self.attributes = attributes
-        self.key_factory = KeyFactory()
         self.validate()
 
     def read(self, istream):
-        super(self.__class__, self).read(istream)
+        super(KeyMaterialStruct, self).read(istream)
         tstream = BytearrayStream(istream.read(self.length))
 
-        self.key_material = self.key_factory.create_key(self.key_format_type)
-        self.key_material.read(tstream)
+        self.data = BytearrayStream(tstream.read())
 
-        self.attributes = list()
+        self.is_oversized(tstream)
+        self.validate()
+
+    def write(self, ostream):
+        tstream = BytearrayStream()
+        tstream.write(self.data.buffer)
+
+        self.length = tstream.length()
+        super(KeyMaterialStruct, self).write(ostream)
+        ostream.write(tstream.buffer)
+
+    def validate(self):
+        self.__validate()
+
+    def __validate(self):
+        # NOTE (peter-hamilton): Intentional pass, no way to validate data.
+        pass
+
+
+class KeyValue(Struct):
+
+    def __init__(self,
+                 key_material=None,
+                 attributes=None):
+        super(KeyValue, self).__init__(Tags.KEY_VALUE)
+
+        if key_material is None:
+            self.key_material = KeyMaterial()
+        else:
+            self.key_material = key_material
+
+        if attributes is None:
+            self.attributes = list()
+        else:
+            self.attributes = attributes
+
+        self.validate()
+
+    def read(self, istream):
+        super(KeyValue, self).read(istream)
+        tstream = BytearrayStream(istream.read(self.length))
+
+        # TODO (peter-hamilton) Replace this with a KeyMaterial factory.
+        if self.is_type_next(Types.STRUCTURE, tstream):
+            self.key_material = KeyMaterialStruct()
+            self.key_material.read(tstream)
+        else:
+            self.key_material = KeyMaterial()
+            self.key_material.read(tstream)
 
-        # Read the attributes, 0 or more
         while self.is_tag_next(Tags.ATTRIBUTE, tstream):
             attribute = Attribute()
             attribute.read(tstream)
@@ -463,69 +527,37 @@ class KeyValueStruct(Struct):
 
         self.key_material.write(tstream)
 
-        if self.attributes is not None:
-            for attribute in self.attributes:
-                attribute.write(tstream)
+        for attribute in self.attributes:
+            attribute.write(tstream)
 
-        # Write the length and value of the credential
         self.length = tstream.length()
-        super(self.__class__, self).write(ostream)
+        super(KeyValue, self).write(ostream)
         ostream.write(tstream.buffer)
 
     def validate(self):
         self.__validate()
 
     def __validate(self):
-        # TODO (peter-hamilton) Finish implementation.
-        pass
+        # TODO (peter-hamilton) Replace with check against KeyMaterial factory.
+        if not isinstance(self.key_material, KeyMaterial):
+            msg = "invalid key material"
+            msg += "; expected {0}, received {1}".format(
+                KeyMaterial, self.key_material)
+            raise TypeError(msg)
 
-
-class KeyValue(Struct):
-    '''
-    KeyValue can be either a ByteString or a Struct. Therefore, this class
-    acts as a wrapper for two different KeyValue objects, KeyValueString,
-    which represents the ByteString format, and KeyValueStruct, which
-    represents the Struct format, both of which are defined above. This
-    KeyValue object does not read or write itself; instead, it reads and
-    writes its internal key_value attribute, which is either a KeyValueString
-    or a KeyValueStruct.
-
-    When reading, the class determines what the format of its internal
-    structure should be by looking at the type of the object it will read
-    using KeyValue.is_type_next(). This is one of the only places in the
-    code where this approach is used.
-    '''
-
-    def __init__(self,
-                 key_value=None,
-                 key_format_type=None):
-        super(self.__class__, self).__init__(Tags.KEY_VALUE)
-        self.key_value = key_value
-        self.key_format_type = key_format_type
-        if self.key_value is not None:
-            self.type = key_value.type
-        self.validate()
-
-    def read(self, istream):
-        if self.is_type_next(Types.BYTE_STRING, istream):
-            self.key_value = KeyValueString()
-            self.key_value.read(istream)
-        elif self.is_type_next(Types.STRUCTURE, istream):
-            kft = self.key_format_type
-            self.key_value = KeyValueStruct(key_format_type=kft)
-            self.key_value.read(istream)
-
-    def write(self, ostream):
-        tstream = BytearrayStream()
-        self.key_value.write(tstream)
-        ostream.write(tstream.buffer)
-
-    def validate(self):
-        self.__validate()
-
-    def __validate(self):
-        # TODO (peter-hamilton) Finish implementation.
-        pass
+        if isinstance(self.attributes, list):
+            for i in xrange(len(self.attributes)):
+                attribute = self.attributes[i]
+                if not isinstance(attribute, Attribute):
+                    msg = "invalid attribute ({0} in list)".format(i)
+                    msg += "; expected {0}, received {1}".format(
+                        Attribute, attribute)
+                    raise TypeError(msg)
+        else:
+            msg = "invalid attributes list"
+            msg += "; expected {0}, received {1}".format(
+                list, self.attributes)
+            raise TypeError(msg)
 
 
 # 2.1.5
@@ -540,7 +572,7 @@ class EncodingOption(Enumeration):
     ENUM_TYPE = enums.EncodingOption
 
     def __init__(self, value=None):
-        super(WrappingMethod, self).__init__(value, Tags.ENCODING_OPTION)
+        super(EncodingOption, self).__init__(value, Tags.ENCODING_OPTION)
 
 
 class KeyInformation(Struct):
@@ -549,14 +581,13 @@ class KeyInformation(Struct):
                  unique_identifier=None,
                  cryptographic_parameters=None,
                  tag=Tags.ENCRYPTION_KEY_INFORMATION):
-        super(self.__class__, self).\
-            __init__(tag=Tags.ENCRYPTION_KEY_INFORMATION)
+        super(KeyInformation, self).__init__(tag=tag)
         self.unique_identifier = unique_identifier
         self.cryptographic_parameters = cryptographic_parameters
         self.validate()
 
     def read(self, istream):
-        super(self.__class__, self).read(istream)
+        super(KeyInformation, self).read(istream)
         tstream = BytearrayStream(istream.read(self.length))
 
         self.unique_identifier = attributes.UniqueIdentifier()
@@ -579,7 +610,7 @@ class KeyInformation(Struct):
 
         # Write the length and value of the template attribute
         self.length = tstream.length()
-        super(self.__class__, self).write(ostream)
+        super(KeyInformation, self).write(ostream)
         ostream.write(tstream.buffer)
 
     def validate(self):
@@ -596,8 +627,8 @@ class EncryptionKeyInformation(KeyInformation):
                  unique_identifier=None,
                  cryptographic_parameters=None,
                  tag=Tags.ENCRYPTION_KEY_INFORMATION):
-        super(self.__class__, self).\
-            __init__(unique_identifier, cryptographic_parameters, tag)
+        super(EncryptionKeyInformation, self).__init__(
+            unique_identifier, cryptographic_parameters, tag)
 
     def validate(self):
         self.__validate()
@@ -613,8 +644,8 @@ class MACSignatureKeyInformation(KeyInformation):
                  unique_identifier=None,
                  cryptographic_parameters=None,
                  tag=Tags.MAC_SIGNATURE_KEY_INFORMATION):
-        super(self.__class__, self).\
-            __init__(unique_identifier, cryptographic_parameters, tag)
+        super(MACSignatureKeyInformation, self).__init__(
+            unique_identifier, cryptographic_parameters, tag)
 
     def validate(self):
         self.__validate()
@@ -629,12 +660,14 @@ class KeyWrappingData(Struct):
     class MACSignature(ByteString):
 
         def __init__(self, value=None):
-            super(self.__class__, self).__init__(value, Tags.MAC_SIGNATURE)
+            super(KeyWrappingData.MACSignature, self).__init__(
+                value, Tags.MAC_SIGNATURE)
 
     class IVCounterNonce(ByteString):
 
         def __init__(self, value=None):
-            super(self.__class__, self).__init__(value, Tags.IV_COUNTER_NONCE)
+            super(KeyWrappingData.IVCounterNonce, self).__init__(
+                value, Tags.IV_COUNTER_NONCE)
 
     def __init__(self,
                  wrapping_method=None,
@@ -643,7 +676,7 @@ class KeyWrappingData(Struct):
                  mac_signature=None,
                  iv_counter_nonce=None,
                  encoding_option=None):
-        super(self.__class__, self).__init__(Tags.KEY_WRAPPING_DATA)
+        super(KeyWrappingData, self).__init__(Tags.KEY_WRAPPING_DATA)
         self.wrapping_method = wrapping_method
         self.encryption_key_information = encryption_key_information
         self.mac_signature_key_information = mac_signature_key_information
@@ -653,7 +686,7 @@ class KeyWrappingData(Struct):
         self.validate()
 
     def read(self, istream):
-        super(self.__class__, self).read(istream)
+        super(KeyWrappingData, self).read(istream)
         tstream = BytearrayStream(istream.read(self.length))
 
         self.wrapping_method = WrappingMethod()
@@ -701,7 +734,7 @@ class KeyWrappingData(Struct):
 
         # Write the length and value of the key wrapping data
         self.length = tstream.length()
-        super(self.__class__, self).write(ostream)
+        super(KeyWrappingData, self).write(ostream)
         ostream.write(tstream.buffer)
 
     def validate(self):
@@ -718,7 +751,8 @@ class KeyWrappingSpecification(Struct):
     class AttributeName(TextString):
 
         def __init__(self, value=None):
-            super(self.__class__, self).__init__(value, Tags.ATTRIBUTE_NAME)
+            super(KeyWrappingSpecification.AttributeName, self).__init__(
+                value, Tags.ATTRIBUTE_NAME)
 
     def __init__(self,
                  wrapping_method=None,
@@ -726,8 +760,8 @@ class KeyWrappingSpecification(Struct):
                  mac_signature_key_information=None,
                  attribute_name=None,
                  encoding_option=None):
-        super(self.__class__, self).\
-            __init__(tag=Tags.KEY_WRAPPING_SPECIFICATION)
+        super(KeyWrappingSpecification, self).__init__(
+            tag=Tags.KEY_WRAPPING_SPECIFICATION)
         self.wrapping_method = wrapping_method
         self.encryption_key_information = encryption_key_information
         self.mac_signature_key_information = mac_signature_key_information
@@ -735,7 +769,7 @@ class KeyWrappingSpecification(Struct):
         self.encoding_option = encoding_option
 
     def read(self, istream):
-        super(self.__class__, self).read(istream)
+        super(KeyWrappingSpecification, self).read(istream)
         tstream = BytearrayStream(istream.read(self.length))
 
         self.wrapping_method = WrappingMethod()
@@ -777,7 +811,7 @@ class KeyWrappingSpecification(Struct):
 
         # Write the length and value of the key wrapping data
         self.length = tstream.length()
-        super(self.__class__, self).write(ostream)
+        super(KeyWrappingSpecification, self).write(ostream)
         ostream.write(tstream.buffer)
 
     def validate(self):
@@ -793,14 +827,24 @@ class TemplateAttribute(Struct):
 
     def __init__(self,
                  names=None,
-                 attributes=None):
-        super(self.__class__, self).__init__(tag=Tags.TEMPLATE_ATTRIBUTE)
-        self.names = names
-        self.attributes = attributes
+                 attributes=None,
+                 tag=Tags.TEMPLATE_ATTRIBUTE):
+        super(TemplateAttribute, self).__init__(tag)
+
+        if names is None:
+            self.names = list()
+        else:
+            self.names = names
+
+        if attributes is None:
+            self.attributes = list()
+        else:
+            self.attributes = attributes
+
         self.validate()
 
     def read(self, istream):
-        super(self.__class__, self).read(istream)
+        super(TemplateAttribute, self).read(istream)
         tstream = BytearrayStream(istream.read(self.length))
 
         self.names = list()
@@ -825,16 +869,14 @@ class TemplateAttribute(Struct):
         tstream = BytearrayStream()
 
         # Write the names and attributes of the template attribute
-        if self.names is not None:
-            for name in self.names:
-                name.write(tstream)
-        if self.attributes is not None:
-            for attribute in self.attributes:
-                attribute.write(tstream)
+        for name in self.names:
+            name.write(tstream)
+        for attribute in self.attributes:
+            attribute.write(tstream)
 
         # Write the length and value of the template attribute
         self.length = tstream.length()
-        super(self.__class__, self).write(ostream)
+        super(TemplateAttribute, self).write(ostream)
         ostream.write(tstream.buffer)
 
     def validate(self):
@@ -843,3 +885,395 @@ class TemplateAttribute(Struct):
     def __validate(self):
         # TODO (peter-hamilton) Finish implementation.
         pass
+
+    def __eq__(self, other):
+        if isinstance(other, TemplateAttribute):
+            if len(self.names) != len(other.names):
+                return False
+            if len(self.attributes) != len(other.attributes):
+                return False
+
+            for i in xrange(len(self.names)):
+                a = self.names[i]
+                b = other.names[i]
+
+                if a != b:
+                    return False
+
+            for i in xrange(len(self.attributes)):
+                a = self.attributes[i]
+                b = other.attributes[i]
+
+                if a != b:
+                    return False
+
+            return True
+        else:
+            return NotImplemented
+
+
+class CommonTemplateAttribute(TemplateAttribute):
+
+    def __init__(self,
+                 names=None,
+                 attributes=None):
+        super(CommonTemplateAttribute, self).__init__(
+            names, attributes, Tags.COMMON_TEMPLATE_ATTRIBUTE)
+
+
+class PrivateKeyTemplateAttribute(TemplateAttribute):
+
+    def __init__(self,
+                 names=None,
+                 attributes=None):
+        super(PrivateKeyTemplateAttribute, self).__init__(
+            names, attributes, Tags.PRIVATE_KEY_TEMPLATE_ATTRIBUTE)
+
+
+class PublicKeyTemplateAttribute(TemplateAttribute):
+
+    def __init__(self,
+                 names=None,
+                 attributes=None):
+        super(PublicKeyTemplateAttribute, self).__init__(
+            names, attributes, Tags.PUBLIC_KEY_TEMPLATE_ATTRIBUTE)
+
+
+# 2.1.9
+class ExtensionName(TextString):
+    """
+    The name of an extended Object.
+
+    A part of ExtensionInformation, specifically identifying an Object that is
+    a custom vendor addition to the KMIP specification. See Section 2.1.9 of
+    the KMIP 1.1 specification for more information.
+
+    Attributes:
+        value: The string data representing the extension name.
+    """
+    def __init__(self, value=''):
+        """
+        Construct an ExtensionName object.
+
+        Args:
+            value (str): The string data representing the extension name.
+                Optional, defaults to the empty string.
+        """
+        super(ExtensionName, self).__init__(value, Tags.EXTENSION_NAME)
+
+
+class ExtensionTag(Integer):
+    """
+    The tag of an extended Object.
+
+    A part of ExtensionInformation. See Section 2.1.9 of the KMIP 1.1
+    specification for more information.
+
+    Attributes:
+        value: The tag number identifying the extended object.
+    """
+    def __init__(self, value=0):
+        """
+        Construct an ExtensionTag object.
+
+        Args:
+            value (int): A number representing the extension tag. Often
+                displayed in hex format. Optional, defaults to 0.
+        """
+        super(ExtensionTag, self).__init__(value, Tags.EXTENSION_TAG)
+
+
+class ExtensionType(Integer):
+    """
+    The type of an extended Object.
+
+    A part of ExtensionInformation, specifically identifying the type of the
+    Object in the specification extension. See Section 2.1.9 of the KMIP 1.1
+    specification for more information.
+
+    Attributes:
+        value: The type enumeration for the extended object.
+    """
+    def __init__(self, value=None):
+        """
+        Construct an ExtensionType object.
+
+        Args:
+            value (Types): A number representing a Types enumeration value,
+                indicating the type of the extended Object. Optional, defaults
+                to None.
+        """
+        super(ExtensionType, self).__init__(value, Tags.EXTENSION_TYPE)
+
+
+class ExtensionInformation(Struct):
+    """
+    A structure describing Objects defined in KMIP specification extensions.
+
+    It is used specifically for Objects with Item Tag values in the Extensions
+    range and appears in responses to Query requests for server extension
+    information. See Sections 2.1.9 and 4.25 of the KMIP 1.1 specification for
+    more information.
+
+    Attributes:
+        extension_name: The name of the extended Object.
+        extension_tag: The tag of the extended Object.
+        extension_type: The type of the extended Object.
+    """
+    def __init__(self, extension_name=None, extension_tag=None,
+                 extension_type=None):
+        """
+        Construct an ExtensionInformation object.
+
+        Args:
+            extension_name (ExtensionName): The name of the extended Object.
+            extension_tag (ExtensionTag): The tag of the extended Object.
+            extension_type (ExtensionType): The type of the extended Object.
+        """
+        super(ExtensionInformation, self).__init__(Tags.EXTENSION_INFORMATION)
+
+        if extension_name is None:
+            self.extension_name = ExtensionName()
+        else:
+            self.extension_name = extension_name
+
+        self.extension_tag = extension_tag
+        self.extension_type = extension_type
+
+        self.validate()
+
+    def read(self, istream):
+        """
+        Read the data encoding the ExtensionInformation object and decode it
+        into its constituent parts.
+
+        Args:
+            istream (Stream): A data stream containing encoded object data,
+                supporting a read method; usually a BytearrayStream object.
+        """
+        super(ExtensionInformation, self).read(istream)
+        tstream = BytearrayStream(istream.read(self.length))
+
+        self.extension_name.read(tstream)
+
+        if self.is_tag_next(Tags.EXTENSION_TAG, tstream):
+            self.extension_tag = ExtensionTag()
+            self.extension_tag.read(tstream)
+        if self.is_tag_next(Tags.EXTENSION_TYPE, tstream):
+            self.extension_type = ExtensionType()
+            self.extension_type.read(tstream)
+
+        self.is_oversized(tstream)
+        self.validate()
+
+    def write(self, ostream):
+        """
+        Write the data encoding the ExtensionInformation object to a stream.
+
+        Args:
+            ostream (Stream): A data stream in which to encode object data,
+                supporting a write method; usually a BytearrayStream object.
+        """
+        tstream = BytearrayStream()
+
+        self.extension_name.write(tstream)
+
+        if self.extension_tag is not None:
+            self.extension_tag.write(tstream)
+        if self.extension_type is not None:
+            self.extension_type.write(tstream)
+
+        self.length = tstream.length()
+        super(ExtensionInformation, self).write(ostream)
+        ostream.write(tstream.buffer)
+
+    def validate(self):
+        """
+        Error check the attributes of the ExtensionInformation object.
+        """
+        self.__validate()
+
+    def __validate(self):
+        if not isinstance(self.extension_name, ExtensionName):
+            msg = "invalid extension name"
+            msg += "; expected {0}, received {1}".format(
+                ExtensionName, self.extension_name)
+            raise TypeError(msg)
+
+        if self.extension_tag is not None:
+            if not isinstance(self.extension_tag, ExtensionTag):
+                msg = "invalid extension tag"
+                msg += "; expected {0}, received {1}".format(
+                    ExtensionTag, self.extension_tag)
+                raise TypeError(msg)
+
+        if self.extension_type is not None:
+            if not isinstance(self.extension_type, ExtensionType):
+                msg = "invalid extension type"
+                msg += "; expected {0}, received {1}".format(
+                    ExtensionType, self.extension_type)
+                raise TypeError(msg)
+
+    def __eq__(self, other):
+        if isinstance(other, ExtensionInformation):
+            if self.extension_name != other.extension_name:
+                return False
+            elif self.extension_tag != other.extension_tag:
+                return False
+            elif self.extension_type != other.extension_type:
+                return False
+            else:
+                return True
+        else:
+            return NotImplemented
+
+    def __ne__(self, other):
+        if isinstance(other, ExtensionInformation):
+            return not (self == other)
+        else:
+            return NotImplemented
+
+    def __repr__(self):
+        name = "extension_name={0}".format(repr(self.extension_name))
+        tag = "extension_tag={0}".format(repr(self.extension_tag))
+        typ = "extension_type={0}".format(repr(self.extension_type))
+        return "ExtensionInformation({0}, {1}, {2})".format(name, tag, typ)
+
+    def __str__(self):
+        return repr(self)
+
+    @classmethod
+    def create(cls, extension_name=None, extension_tag=None,
+               extension_type=None):
+        """
+        Construct an ExtensionInformation object from provided extension
+        values.
+
+        Args:
+            extension_name (str): The name of the extension. Optional,
+                defaults to None.
+            extension_tag (int): The tag number of the extension. Optional,
+                defaults to None.
+            extension_type (int): The type index of the extension. Optional,
+                defaults to None.
+
+        Returns:
+            ExtensionInformation: The newly created set of extension
+                information.
+
+        Example:
+            >>> x = ExtensionInformation.create('extension', 1, 1)
+            >>> x.extension_name.value
+            ExtensionName(value='extension')
+            >>> x.extension_tag.value
+            ExtensionTag(value=1)
+            >>> x.extension_type.value
+            ExtensionType(value=1)
+        """
+        extension_name = ExtensionName(extension_name)
+        extension_tag = ExtensionTag(extension_tag)
+        extension_type = ExtensionType(extension_type)
+
+        return ExtensionInformation(
+            extension_name=extension_name,
+            extension_tag=extension_tag,
+            extension_type=extension_type)
+
+
+# 3.31, 9.1.3.2.19
+class RevocationReasonCode(Enumeration):
+    ENUM_TYPE = RevocationReasonCodeEnum
+
+    def __init__(self, value=RevocationReasonCodeEnum.UNSPECIFIED):
+        super(RevocationReasonCode, self).__init__(
+            value=value, tag=Tags.REVOCATION_REASON_CODE)
+
+
+# 3.31
+class RevocationReason(Struct):
+    """
+    A structure describing  the reason for a revocation operation.
+
+    See Sections 2.1.9 and 4.25 of the KMIP 1.1 specification for
+    more information.
+
+    Attributes:
+        code: The revocation reason code enumeration
+        message: An optional revocation message
+    """
+
+    def __init__(self, code=None, message=None):
+        """
+        Construct a RevocationReason object.
+
+        Parameters:
+            code(RevocationReasonCode): revocation reason code
+            message(string): An optional revocation message
+        """
+        super(RevocationReason, self).__init__(tag=Tags.REVOCATION_REASON)
+        if code is not None:
+            self.revocation_code = RevocationReasonCode(value=code)
+        else:
+            self.revocation_code = RevocationReasonCode()
+
+        if message is not None:
+            self.revocation_message = TextString(
+                value=message,
+                tag=Tags.REVOCATION_MESSAGE)
+        else:
+            self.revocation_message = None
+
+        self.validate()
+
+    def read(self, istream):
+        """
+        Read the data encoding the RevocationReason object and decode it
+        into its constituent parts.
+
+        Args:
+            istream (Stream): A data stream containing encoded object data,
+                supporting a read method; usually a BytearrayStream object.
+        """
+        super(RevocationReason, self).read(istream)
+        tstream = BytearrayStream(istream.read(self.length))
+
+        self.revocation_code = RevocationReasonCode()
+        self.revocation_code.read(tstream)
+
+        if self.is_tag_next(Tags.REVOCATION_MESSAGE, tstream):
+            self.revocation_message = TextString()
+            self.revocation_message.read(tstream)
+
+        self.is_oversized(tstream)
+        self.validate()
+
+    def write(self, ostream):
+        """
+        Write the data encoding the RevocationReason object to a stream.
+
+        Args:
+            ostream (Stream): A data stream in which to encode object data,
+                supporting a write method; usually a BytearrayStream object.
+        """
+        tstream = BytearrayStream()
+
+        self.revocation_code.write(tstream)
+        if self.revocation_message is not None:
+            self.revocation_message.write(tstream)
+
+        # Write the length and value
+        self.length = tstream.length()
+        super(RevocationReason, self).write(ostream)
+        ostream.write(tstream.buffer)
+
+    def validate(self):
+        """
+        validate the RevocationReason object
+        """
+        if not isinstance(self.revocation_code, RevocationReasonCode):
+            msg = "RevocationReaonCode expected"
+            raise TypeError(msg)
+        if self.revocation_message is not None:
+            if not isinstance(self.revocation_message, TextString):
+                msg = "TextString expect"
+                raise TypeError(msg)
