@@ -29,16 +29,16 @@ from kmip.core.enums import ResultStatus as RS
 from kmip.core.factories.attributes import AttributeFactory
 from kmip.core.factories.keys import KeyFactory
 from kmip.core.factories.secrets import SecretFactory
-from kmip.core.keys import RawKey
 
-from kmip.core.messages.contents import KeyFormatType
 from kmip.core.messages.contents import ResultStatus
 from kmip.core.messages.contents import ResultReason
 from kmip.core.messages.contents import ResultMessage
 
+from kmip.core.misc import KeyFormatType
+
 from kmip.core.objects import KeyBlock
+from kmip.core.objects import KeyMaterial
 from kmip.core.objects import KeyValue
-from kmip.core.objects import KeyValueStruct
 from kmip.core.objects import TemplateAttribute
 from kmip.core.repo.mem_repo import MemRepo
 from kmip.core.secrets import SymmetricKey
@@ -56,29 +56,40 @@ class KMIP(object):
         pass
 
     def create(self, object_type, template_attribute, credential=None):
-        raise NotImplementedError
+        raise NotImplementedError()
+
+    def create_key_pair(self, common_template_attribute,
+                        private_key_template_attribute,
+                        public_key_template_attribute):
+        raise NotImplementedError()
 
     def register(self, object_type, template_attribute, secret,
                  credential=None):
-        raise NotImplementedError
+        raise NotImplementedError()
+
+    def rekey_key_pair(self, private_key_unique_identifier,
+                       offset, common_template_attribute,
+                       private_key_template_attribute,
+                       public_key_template_attribute):
+        raise NotImplementedError()
 
     def get(self, uuid=None, key_format_type=None, key_compression_type=None,
             key_wrapping_specification=None, credential=None):
-        raise NotImplementedError
+        raise NotImplementedError()
 
     def destroy(self, uuid, credential=None):
-        raise NotImplementedError
+        raise NotImplementedError()
 
     def locate(self, maximum_items=None, storate_status_mask=None,
                object_group_member=None, attributes=None,
                credential=None):
-        raise NotImplementedError
+        raise NotImplementedError()
 
 
 class KMIPImpl(KMIP):
 
     def __init__(self):
-        super(self.__class__, self).__init__()
+        super(KMIPImpl, self).__init__()
         self.logger = logging.getLogger(__name__)
         self.key_factory = KeyFactory()
         self.secret_factory = SecretFactory()
@@ -129,11 +140,15 @@ class KMIPImpl(KMIP):
         s_uuid, uuid_attribute = self._save(key, attributes)
         ret_attributes.append(uuid_attribute)
         template_attribute = TemplateAttribute(attributes=ret_attributes)
-        return CreateResult(ResultStatus(RS.SUCCESS),
-                            object_type=object_type,
+        return CreateResult(ResultStatus(RS.SUCCESS), object_type=object_type,
                             uuid=UniqueIdentifier(s_uuid),
                             template_attribute=template_attribute)
 
+    def create_key_pair(self, common_template_attribute,
+                        private_key_template_attribute,
+                        public_key_template_attribute):
+        raise NotImplementedError()
+
     def register(self, object_type, template_attribute, secret,
                  credential=None):
         self.logger.debug('register() called')
@@ -183,6 +198,12 @@ class KMIPImpl(KMIP):
                               uuid=UniqueIdentifier(s_uuid),
                               template_attribute=template_attribute)
 
+    def rekey_key_pair(self, private_key_unique_identifier,
+                       offset, common_template_attribute,
+                       private_key_template_attribute,
+                       public_key_template_attribute):
+        raise NotImplementedError()
+
     def get(self,
             uuid=None,
             key_format_type=None,
@@ -229,10 +250,8 @@ class KMIPImpl(KMIP):
         # currently only symmetric keys are supported, fix this in future
         object_type = ObjectType(OT.SYMMETRIC_KEY)
         ret_value = RS.SUCCESS
-        return GetResult(ResultStatus(ret_value),
-                         object_type=object_type,
-                         uuid=uuid,
-                         secret=managed_object)
+        return GetResult(ResultStatus(ret_value), object_type=object_type,
+                         uuid=uuid, secret=managed_object)
 
     def destroy(self, uuid):
         self.logger.debug('destroy() called')
@@ -268,8 +287,7 @@ class KMIPImpl(KMIP):
             msg = ResultMessage('Locate Operation Not Supported')
             reason = ResultReason(ResultReasonEnum.OPERATION_NOT_SUPPORTED)
             return LocateResult(ResultStatus(RS.OPERATION_FAILED),
-                                result_reason=reason,
-                                result_message=msg)
+                                result_reason=reason, result_message=msg)
 
     def _validate_req_field(self, attrs, name, expected, msg, required=True):
         self.logger.debug('Validating attribute %s' % name)
@@ -326,9 +344,9 @@ class KMIPImpl(KMIP):
         return OperationResult(status, reason, message)
 
     def _gen_symmetric_key(self, bit_length, crypto_alg):
-        key_format_type = KeyBlock.KeyFormatType(KeyFormatTypeEnum.RAW)
-        key_material = RawKey(bytearray(os.urandom(int(bit_length/8))))
-        key_value = KeyValueStruct(key_format_type, key_material)
+        key_format_type = KeyFormatType(KeyFormatTypeEnum.RAW)
+        key_material = KeyMaterial(os.urandom(int(bit_length/8)))
+        key_value = KeyValue(key_material)
         crypto_length = CryptographicLength(bit_length)
         key_block = KeyBlock(key_format_type, None, key_value, crypto_alg,
                              crypto_length, None)
@@ -366,7 +384,6 @@ class KMIPImpl(KMIP):
             kv = key_block.key_value
             if isinstance(kv, KeyValue):
                 kv = key_block.key_value
-            if isinstance(kv, KeyValueStruct):
                 if kv.attributes is not None:
                     self.logger.debug('adding the key value struct attributes')
                     attributes.extend(kv.attributes)
@@ -376,5 +393,5 @@ class KMIPImpl(KMIP):
 class InvalidFieldException(Exception):
 
     def __init__(self, result):
-        super(self.__class__, self).__init__()
+        super(InvalidFieldException, self).__init__()
         self.result = result
